@@ -23,6 +23,7 @@ import (
 	"fmt"
 	"io"
 	"net/http"
+	"strings"
 )
 
 /* **** SAMPLE ERROR RESPONSE ****
@@ -188,6 +189,15 @@ func httpRespToErrorResponse(resp *http.Response, bucketName, objectName string)
 		}
 	}
 
+	code := resp.Header.Get("x-minio-error-code")
+	if code != "" {
+		errResp.Code = code
+	}
+	desc := resp.Header.Get("x-minio-error-desc")
+	if desc != "" {
+		errResp.Message = strings.Trim(desc, `"`)
+	}
+
 	// Save hostID, requestID and region information
 	// from headers if not available through error XML.
 	if errResp.RequestID == "" {
