@@ -34,6 +34,7 @@ var awsS3EndpointMap = map[string]string{
 	"eu-south-2":     "s3.dualstack.eu-south-2.amazonaws.com",
 	"ap-east-1":      "s3.dualstack.ap-east-1.amazonaws.com",
 	"ap-south-1":     "s3.dualstack.ap-south-1.amazonaws.com",
+	"ap-south-2":     "s3.dualstack.ap-south-2.amazonaws.com",
 	"ap-southeast-1": "s3.dualstack.ap-southeast-1.amazonaws.com",
 	"ap-southeast-2": "s3.dualstack.ap-southeast-2.amazonaws.com",
 	"ap-northeast-1": "s3.dualstack.ap-northeast-1.amazonaws.com",
@@ -48,6 +49,8 @@ var awsS3EndpointMap = map[string]string{
 	"cn-north-1":     "s3.dualstack.cn-north-1.amazonaws.com.cn",
 	"cn-northwest-1": "s3.dualstack.cn-northwest-1.amazonaws.com.cn",
 	"ap-southeast-3": "s3.dualstack.ap-southeast-3.amazonaws.com",
+	"ap-southeast-4": "s3.dualstack.ap-southeast-4.amazonaws.com",
+	"il-central-1":   "s3.dualstack.il-central-1.amazonaws.com",
 }
 
 // getS3Endpoint get Amazon S3 endpoint based on the bucket location.
